#include "ledctrl.h"
#include <string.h>

void ledInit(ledCtrlType* led,char lowActive) {
    led->lowActive=lowActive;
    led->invertFlash=0;
    led->state=~lowActive;  // always init to off state
    led->currentBlinkTime=0;
    led->currentFlashTime=0;
}

void ledSetFlashTime(ledCtrlType* led,unsigned long flashTime) {
    led->flashTime=flashTime;    
}

__bit ledOn(ledCtrlType* led) {
    led->state=1;
    led->currentBlinkTime=0;
    led->currentFlashTime=0;
    return (__bit)!led->lowActive;
}

__bit ledOff(ledCtrlType* led) {
    led->state=0;
    led->currentBlinkTime=0;
    led->currentFlashTime=0;
    return (__bit)led->lowActive;
}

void ledTriggerFlashInvert(ledCtrlType* led) {
    led->currentFlashTime=led->flashTime;
    led->invertFlash=1;
}

void ledTriggerFlash(ledCtrlType* led) {
    led->currentFlashTime=led->flashTime;
    led->invertFlash=0;
}

void ledSetBlinkTimes(ledCtrlType* led,unsigned long on,unsigned long off) {
    led->blinkOffTime=off; 
    led->blinkOnTime=on;
}
    
void ledTriggerBlink(ledCtrlType* led) {
    led->currentBlinkTime=led->blinkOnTime;
}

__bit ledState(ledCtrlType* led) {
    return (__bit)(led->state ^ led->lowActive);
}

void ledHop(ledCtrlType* led, unsigned long hop) {
    if (led->currentFlashTime>hop)
        led->currentFlashTime-=hop;
    else
        led->currentFlashTime=0; 
    if (led->currentBlinkTime>hop)
        led->currentBlinkTime-=hop;
    else
        led->currentBlinkTime=0;        
}

__bit  ledTick(ledCtrlType* led) {
    if (led->currentFlashTime) {
        led->currentFlashTime--;
        led->state=1;
        return (__bit)((led->state ^ led->lowActive) ^ led->invertFlash);
    } else {
        led->state=0;
        return (__bit)((led->state ^ led->lowActive) ^ led->invertFlash);
    }
    
    if (led->currentBlinkTime) {
        led->currentBlinkTime--;
    } else {
        if (led->state) {
            led->currentBlinkTime=led->blinkOffTime;
            led->state=0;
        } else {
            led->currentBlinkTime=led->blinkOnTime;
            led->state=1;
        }
        return (__bit)(led->state ^ led->lowActive);
    }
    
    return (__bit)(led->state ^ led->lowActive);
}


