#include "mcc_generated_files/mcc.h"
#include "spi.h"

unsigned short spiSendCsr;
unsigned short spiRecvCsr;
        
spiPacket* spiActive;
spiPacket* spiSend;
spiPacket spiTmit1;
spiPacket spiTmit2;
spiPacket spiRecv;

static void (*spiTransferComplete)(spiTransmitPacket*);

char switchSpiBuffer()
{
    char dma;
    if (spiActive==&spiTmit1) {
        spiActive=&spiTmit2;
        spiSend=&spiTmit1;
        dma=1;
    } else {
        spiActive=&spiTmit1;
        spiSend=&spiTmit2;
        dma=2;
    }
    spiActive->pck.U1len=0;
    spiActive->pck.U2len=0;
    spiActive->pck.U3len=0;
    spiActive->pck.U4len=0;
    return dma;
}

void spiInit() {
    unsigned short test;
        
    spiTransferComplete=0;
    // Chip not ready
    CR=1;
    CRDAT=0;
    
    // clear EN 
    GIE=0;
    SPI1CON0bits.EN=0;
    ANSELA=0;
    ANSELB=0;
    ANSELC=0;
    
    // Set PPS     
    PPSLOCK=0x55;
    PPSLOCK=0xAA;
    PPSLOCKbits.PPSLOCKED=0; 
    
    SPI1SSPPS=0b00000110; // SS  RA6 > Chip select input            
    SPI1SCKPPS=0b00001010; // SCK > RB2
    SPI1SDIPPS=0b00001011; // SDI > RB3
    RB4PPS=0x32; // SDO > RB4 
    TRISB2=1;
    TRISB3=1;    
    TRISA6=1;
    
    PPSLOCK=0x55;
    PPSLOCK=0xAA;
    PPSLOCKbits.PPSLOCKED=1; 
    
    SPI1CON0=0b00000000; // EN=0, LSBF=0, MST=0, BMODE=0
    SPI1CON1=0b00100100; // SMP=0, CKE=0, CKP=0, SSP=1, SDI=0, SDO=0
    SPI1CON2=0b00000011; // SSET=0, TXR=1, RXR=1
    SPI1STATUSbits.CLRBF=1; // Clear buffers
    
    // RX DMA
    DMASELECT=0; // select channel 1
    DMAnSSA=&SPI1RXB; // source reigster is SPI1RXB
    DMAnSSZ=0x01; // Message size = 1            
    DMAnSIRQ=0x18; // SPI1RX 
    DMAnDSA=&(spiRecv.cBuf);
    DMAnDSZ=0x100; // 256 byte
    DMAnCON1=0b01100000; // DMODE=+1; DSTP=1; SMR=00; SMODE=00;SSTP=0
    DMAnCON0=0b10000000; // EN=1; SIRQEN=0; DGO=0; AIRQEN=0;XIP=0;
    
    // TX1 DMA
    DMASELECT=1;
    DMAnSSA=&(spiTmit1.cBuf)+1;
    DMAnSSZ=0xff;
    DMAnSIRQ=0x19;
    DMAnDSA=&SPI1TXB;
    DMAnDSZ=1;
    DMAnCON1=0b00000011;
    DMAnCON0=0b10000000; // EN=1; SIRQEN=0; DGO=0; AIRQEN=0;XIP=0;
    
    // TX2 DMA
    DMASELECT=2;
    DMAnSSA=&(spiTmit2.cBuf)+1;
    DMAnSSZ=0xff;
    DMAnSIRQ=0x19;
    DMAnDSA=&SPI1TXB;
    DMAnDSZ=1;
    DMAnCON1=0b00000011;
    DMAnCON0=0b10000000; // EN=1; SIRQEN=0; DGO=0; AIRQEN=0;XIP=0;
    
    PRLOCK=0x55;
    PRLOCK=0xAA;
    PRLOCKbits.PRLOCKED=0;
    DMA1PR=1;    
    PRLOCK=0x55;
    PRLOCK=0xAA;
    PRLOCKbits.PRLOCKED=1;
      
    // Interrupts
    SPI1INTEbits.SOSIE=1;
    SPI1INTEbits.EOSIE=1;
    SPI1INTEbits.SPI1EOSIE=1;
    SPI1INTEbits.SPI1SOSIE=1;
    
    PIE3bits.SPI1IE=1;
    
    // enable EN 
    SPI1CON0bits.EN=1;
    GIE=1;
}

void spiSetTranferCompleteHandler(void (*spiTrCompl)(spiTransmitPacket*) ) {
    spiTransferComplete=spiTrCompl;
}

void spiInterruptHandler() {
    char dma;
    // Start SS           
    if (SPI1INTEbits.SPI1SOSIE==1 && SPI1INTFbits.SPI1SOSIF==1) {
        dma=switchSpiBuffer(); // prepare sending
        
        spiSendCsr=1;
        spiRecvCsr=0; 
        if (spiSend->pck.U1len!=0
                || spiSend->pck.U2len!=0
                || spiSend->pck.U3len!=0
                || spiSend->pck.U4len!=0)
        {
            spiRecv.pck.U1len=0; 
            spiRecv.pck.U2len=0;
            spiRecv.pck.U3len=0;
            spiRecv.pck.U4len=0;
        }
        
        SPI1STATUSbits.CLRBF=1;        
        SPI1TXB=spiSend->cBuf[0]; // preload first byte        
        
        DMASELECT=0;
        DMAnCON0bits.SIRQEN=1; // ready RX DMA
                
        DMASELECT=dma;
        DMAnCON0bits.SIRQEN=1; // ready TX DMA
        
        CR=0;   // ready to send
        SPI1INTFbits.SPI1SOSIF=0;       
    }
    
    // end of SS
    if (SPI1INTEbits.SPI1EOSIE==1 && SPI1INTFbits.SPI1EOSIF==1) {        
        CR=1;                
        if (spiTransferComplete)
            spiTransferComplete(&(spiRecv.pck));        
        SPI1INTFbits.SPI1EOSIF=0;
    }    
}



