#include "spipacket.h"
#include "ledctrl.h"
#include "leds.h"
#include "fifo.h"

ledCtrlType ledU1;
ledCtrlType ledU2;
ledCtrlType ledU3;
ledCtrlType ledU4;  

unsigned char txBuf[4][MAXPACKETSIZE];
fifoType txFifo[4];

void U1_RxDataHandler(void){
    // use this default receive interrupt handler code
    if (spiActive->pck.U1len<MAXPACKETSIZE)
        spiActive->pck.U1data[spiActive->pck.U1len++]=U1RXB;  
    ledTriggerFlash(&ledU1);
}
void U2_RxDataHandler(void){
    // use this default receive interrupt handler code
    if (spiActive->pck.U2len<MAXPACKETSIZE)
        spiActive->pck.U2data[spiActive->pck.U2len++]=U2RXB;
    ledTriggerFlash(&ledU2);
}
void U3_RxDataHandler(void){
    // use this default receive interrupt handler code
    if (spiActive->pck.U3len<MAXPACKETSIZE)
        spiActive->pck.U3data[spiActive->pck.U3len++]=U4RXB;
    ledTriggerFlash(&ledU3);
}
void U4_RxDataHandler(void){
    // use this default receive interrupt handler code
    if (spiActive->pck.U4len<MAXPACKETSIZE)
        spiActive->pck.U4data[spiActive->pck.U4len++]=U5RXB;
    ledTriggerFlash(&ledU4);
}

void U1_TxDataHandler(void){
    char tx;
    if (fifoGet(&txFifo[0],&tx))
        U1TXB=tx;
    else
        PIE4bits.U1TXIE=0;
    ledTriggerFlash(&ledU1);
}
void U2_TxDataHandler(void){
    char tx;
    if (fifoGet(&txFifo[1],&tx))
        U2TXB=tx;
    else
        PIE8bits.U2TXIE=0;
    ledTriggerFlash(&ledU2);
}
void U3_TxDataHandler(void){
    char tx;
    if (fifoGet(&txFifo[2],&tx))
        U4TXB=tx;
    else
        PIE12bits.U4TXIE=0;
    ledTriggerFlash(&ledU3);
}
void U4_TxDataHandler(void){
    char tx;
    if (fifoGet(&txFifo[3],&tx))
        U5TXB=tx;
    else
        PIE13bits.U5TXIE=0;
    ledTriggerFlash(&ledU4);
}

void copyToFifos(spiTransmitPacket* pck)
{
    if (pck->U1len) {
        fifoPutBuf(&txFifo[0],pck->U1data,pck->U1len);
        PIE4bits.U1TXIE=1;
    }
    if (pck->U2len) {
        fifoPutBuf(&txFifo[1],pck->U2data,pck->U2len);
        PIE8bits.U2TXIE=1;
    }
    if (pck->U3len) {
        fifoPutBuf(&txFifo[2],pck->U3data,pck->U3len);
        PIE12bits.U4TXIE=1;
    }
    if (pck->U4len) {
        fifoPutBuf(&txFifo[3],pck->U4data,pck->U4len);
        PIE13bits.U5TXIE=1;
    }
}

void uartsInit() {
    UART1_Initialize();
    UART2_Initialize();
    UART4_Initialize();
    UART5_Initialize();
    
    fifoInit(&txFifo[0],txBuf[0],MAXPACKETSIZE);
    fifoInit(&txFifo[1],txBuf[1],MAXPACKETSIZE);
    fifoInit(&txFifo[2],txBuf[2],MAXPACKETSIZE);
    fifoInit(&txFifo[3],txBuf[3],MAXPACKETSIZE);
    
    UART1_SetRxInterruptHandler(U1_RxDataHandler);
    UART1_SetTxInterruptHandler(U1_TxDataHandler);
    UART2_SetRxInterruptHandler(U2_RxDataHandler);
    UART2_SetTxInterruptHandler(U2_TxDataHandler);
    UART4_SetRxInterruptHandler(U3_RxDataHandler);
    UART4_SetTxInterruptHandler(U3_TxDataHandler);
    UART5_SetRxInterruptHandler(U4_RxDataHandler);
    UART5_SetTxInterruptHandler(U4_TxDataHandler);    
}